@@ -36,6 +36,8 @@ OF_ASSUME_NONNULL_BEGIN
 
 #ifdef OF_HAVE_CLASS_PROPERTIES
 @property (class, readonly, nonatomic) OFCharacterSet *whitespaceCharacterSet;
+@property (class, readonly, nonatomic) OFCharacterSet *newlineCharacterSet;
+@property (class, readonly, nonatomic) OFCharacterSet *controlCharacterSet;
 #endif
 
 /**
@@ -68,6 +70,18 @@ OF_ASSUME_NONNULL_BEGIN
  */
 + (OFCharacterSet *)whitespaceCharacterSet;
 
+/**
+ * @brief A character set containing U+000A to U+000D, U+0085, U+2028 and
+ *	  U+2029.
+ */
++ (OFCharacterSet *)newlineCharacterSet;
+
+/**
+ * @brief A character set containing all Unicode characters in the category
+ *	  `Cc`.
+ */
++ (OFCharacterSet *)controlCharacterSet;
+
 /**
  * @brief Initializes an already allocated character set with the characters of
  *	  the specified string.
