@@ -21,24 +21,21 @@
 
 #import "OFConcreteColor.h"
 
-#import "OFInvalidArgumentException.h"
-
 @implementation OFConcreteColor
 - (instancetype)initWithRed: (float)red
 		      green: (float)green
 		       blue: (float)blue
 		      alpha: (float)alpha
+		 colorSpace: (OFColorSpace *)colorSpace
 {
 	self = [super init];
 
 	@try {
-		if (alpha < 0.0 || alpha > 1.0)
-			@throw [OFInvalidArgumentException exception];
-
 		_red = red;
 		_green = green;
 		_blue = blue;
 		_alpha = alpha;
+		_colorSpace = objc_retain(colorSpace);
 	} @catch (id e) {
 		objc_release(self);
 		@throw e;
@@ -47,6 +44,13 @@
 	return self;
 }
 
+- (void)dealloc
+{
+	objc_release(_colorSpace);
+
+	[super dealloc];
+}
+
 - (void)getRed: (float *)red
 	 green: (float *)green
 	  blue: (float *)blue
@@ -59,4 +63,9 @@
 	if (alpha != NULL)
 		*alpha = _alpha;
 }
+
+- (OFColorSpace *)colorSpace
+{
+	return _colorSpace;
+}
 @end
