@@ -39,7 +39,7 @@ struct {
 	struct class *isa, *superclass;
 	const char *name;
 	long version, info, instanceSize;
-	struct ivar_list *iVars;
+	struct ivar_list *ivars;
 	struct method_list **methodList;
 	struct cache *cache;
 	struct protocol_list *protocols;
