@@ -103,7 +103,7 @@
 # include <proto/locale.h>
 # undef Class
 # ifdef OF_AMIGAOS4
-#  define DeleteFile(path) Delete(path)
+#  include <dos/obsolete.h>
 # endif
 #endif
 
@@ -111,7 +111,29 @@
 # include <dos/dostags.h>
 #endif
 
-#if defined(OF_WINDOWS) || (defined(OF_AMIGAOS) && !defined(OF_MORPHOS))
+#ifdef DJGPP
+# include <io.h>
+#endif
+
+#if defined(OF_WINDOWS) || defined(OF_AMIGAOS)
+# ifdef st_atime
+#  undef st_atime
+# endif
+# ifdef st_mtime
+#  undef st_mtime
+# endif
+# ifdef st_ctime
+#  undef st_ctime
+# endif
+# ifdef HAVE_STRUCT_STAT_ST_ATIM
+#  undef HAVE_STRUCT_STAT_ST_ATIM
+# endif
+# ifdef HAVE_STRUCT_STAT_ST_MTIM
+#  undef HAVE_STRUCT_STAT_ST_MTIM
+# endif
+# ifdef HAVE_STRUCT_STAT_ST_CTIM
+#  undef HAVE_STRUCT_STAT_ST_CTIM
+# endif
 typedef struct {
 	OFStreamOffset st_size;
 	unsigned int st_mode;
@@ -121,6 +143,11 @@ typedef struct {
 	OFTimeInterval st_birthtime;
 	DWORD fileAttributes;
 # endif
+# ifdef OF_AMIGAOS
+	uint32_t st_uid, st_gid, protection;
+	/* TODO: Allow more on AmigaOS 4? */
+	char comment[80];
+# endif
 } Stat;
 #elif defined(HAVE_STAT64)
 typedef struct stat64 Stat;
@@ -133,6 +160,23 @@ typedef struct stat Stat;
 # define S_ISLNK(mode) (mode & S_IFLNK)
 #endif
 
+#if !defined(HAVE_STRUCT_STAT_ST_ATIM) && defined(HAVE_STRUCT_STAT_ST_ATIMESPEC)
+# define st_atim st_atimespec
+#endif
+#if !defined(HAVE_STRUCT_STAT_ST_MTIM) && defined(HAVE_STRUCT_STAT_ST_MTIMESPEC)
+# define st_mtim st_mtimespec
+#endif
+#if !defined(HAVE_STRUCT_STAT_ST_CTIM) && defined(HAVE_STRUCT_STAT_ST_CTIMESPEC)
+# define st_ctim st_ctimespec
+#endif
+
+#ifndef UID_MAX
+# define UID_MAX ((unsigned long)~(uid_t)0)
+#endif
+#ifndef GID_MAX
+# define GID_MAX ((unsigned long)~(gid_t)0)
+#endif
+
 #if defined(OF_FILE_MANAGER_SUPPORTS_OWNER) && defined(OF_HAVE_THREADS)
 static OFMutex *passwdMutex;
 
@@ -320,7 +364,7 @@ statWrapper(OFString *path, Stat *buffer)
 #  ifdef EX64TAG_PosixDate
 		{ EX64TAG_PosixDate, TRUE },
 #  endif
-		{ TAG_DONE, 0 }
+		{ TAG_END, 0 }
 	};
 
 #  ifdef FIBEXTF_POSIXDATE
@@ -343,7 +387,7 @@ statWrapper(OFString *path, Stat *buffer)
 # if defined(OF_MORPHOS)
 	buffer->st_size = fib.fib_Size64;
 # elif defined(OF_AMIGAOS4)
-	buffer->st_size = ed->FileSize;
+	buffer->st_size = (ed->FileSize != -1 ? ed->FileSize : 0);
 # else
 	buffer->st_size = fib.fib_Size;
 # endif
@@ -353,12 +397,47 @@ statWrapper(OFString *path, Stat *buffer)
 	buffer->st_mode = (fib.fib_DirEntryType > 0 ? S_IFDIR : S_IFREG);
 # endif
 
+# ifdef OF_AMIGAOS4
+	buffer->st_uid = ed->OwnerUID;
+	buffer->st_gid = ed->OwnerGID;
+	buffer->protection = ed->Protection;
+
+	if (ed->CommentSize > 80) {
+		memcpy(buffer->comment, ed->Comment, 79);
+		buffer->comment[79] = '\0';
+	} else
+		memcpy(buffer->comment, ed->Comment, ed->CommentSize);
+# else
+	buffer->st_uid = fib.fib_OwnerUID;
+	buffer->st_gid = fib.fib_OwnerGID;
+	buffer->protection = fib.fib_Protection;
+	memcpy(buffer->comment, fib.fib_Comment, sizeof(buffer->comment));
+# endif
+
+	if (!(buffer->protection & FIBF_READ))
+		buffer->st_mode |= 0400;
+	if (!(buffer->protection & FIBF_WRITE))
+		buffer->st_mode |= 0200;
+	if (!(buffer->protection & FIBF_EXECUTE))
+		buffer->st_mode |= 0100;
+	if (buffer->protection & FIBF_GRP_READ)
+		buffer->st_mode |= 0040;
+	if (buffer->protection & FIBF_GRP_WRITE)
+		buffer->st_mode |= 0020;
+	if (buffer->protection & FIBF_GRP_EXECUTE)
+		buffer->st_mode |= 0010;
+	if (buffer->protection & FIBF_OTR_READ)
+		buffer->st_mode |= 0004;
+	if (buffer->protection & FIBF_OTR_WRITE)
+		buffer->st_mode |= 0002;
+	if (buffer->protection & FIBF_OTR_EXECUTE)
+		buffer->st_mode |= 0001;
+
 # if defined(OF_MORPHOS) && defined(FIBEXTF_POSIXDATE)
 	if (fib.fib_ActExtFlags & FIBEXTF_POSIXDATE)
 		timeInterval = fib.fib_PosixDate.pds_Sec;
 	else {
 # endif
-
 		timeInterval = 252460800;	/* 1978-01-01 */
 
 		locale = OpenLocale(NULL);
@@ -493,17 +572,38 @@ setTypeAttribute(OFMutableFileAttributes attributes, Stat *s)
 static void
 setDateAttributes(OFMutableFileAttributes attributes, Stat *s)
 {
-	/* FIXME: We could be more precise on some OSes */
-	[attributes
-	    setObject: [OFDate dateWithTimeIntervalSince1970: s->st_atime]
-	       forKey: OFFileLastAccessDate];
-	[attributes
-	    setObject: [OFDate dateWithTimeIntervalSince1970: s->st_mtime]
-	       forKey: OFFileModificationDate];
-	[attributes
-	    setObject: [OFDate dateWithTimeIntervalSince1970: s->st_ctime]
-	       forKey: OFFileStatusChangeDate];
-#ifdef HAVE_STRUCT_STAT_ST_BIRTHTIME
+	OFDate *date;
+
+#ifdef HAVE_STRUCT_STAT_ST_ATIM
+	date = [OFDate dateWithTimeIntervalSince1970: s->st_atim.tv_sec +
+	    (OFTimeInterval)s->st_atim.tv_nsec / 1000000000];
+#else
+	date = [OFDate dateWithTimeIntervalSince1970: s->st_atime];
+#endif
+	[attributes setObject: date forKey: OFFileLastAccessDate];
+
+#ifdef HAVE_STRUCT_STAT_ST_MTIM
+	date = [OFDate dateWithTimeIntervalSince1970: s->st_mtim.tv_sec +
+	    (OFTimeInterval)s->st_mtim.tv_nsec / 1000000000];
+#else
+	date = [OFDate dateWithTimeIntervalSince1970: s->st_mtime];
+#endif
+	[attributes setObject: date forKey: OFFileModificationDate];
+
+#ifdef HAVE_STRUCT_STAT_ST_CTIM
+	date = [OFDate dateWithTimeIntervalSince1970: s->st_ctim.tv_sec +
+	    (OFTimeInterval)s->st_ctim.tv_nsec / 1000000000];
+#else
+	date = [OFDate dateWithTimeIntervalSince1970: s->st_ctime];
+#endif
+	[attributes setObject: date forKey: OFFileStatusChangeDate];
+
+#if defined(HAVE_STRUCT_STAT_ST_BIRTHTIMESPEC)
+	date = [OFDate dateWithTimeIntervalSince1970:
+	    s->st_birthtimespec.tv_sec +
+	    (OFTimeInterval)s->st_birthtimespec.tv_nsec / 1000000000];
+	[attributes setObject: date forKey: OFFileCreationDate];
+#elif defined(HAVE_STRUCT_STAT_ST_BIRTHTIME)
 	[attributes
 	    setObject: [OFDate dateWithTimeIntervalSince1970: s->st_birthtime]
 	       forKey: OFFileCreationDate];
@@ -519,10 +619,11 @@ setOwnerAndGroupAttributes(OFMutableFileAttributes attributes, Stat *s)
 	[attributes setObject: [OFNumber numberWithUnsignedLong: s->st_gid]
 		       forKey: OFFileGroupOwnerAccountID];
 
-# ifdef OF_HAVE_THREADS
+# ifndef OF_AMIGAOS
+#  ifdef OF_HAVE_THREADS
 	[passwdMutex lock];
 	@try {
-# endif
+#  endif
 		OFStringEncoding encoding = [OFLocale encoding];
 		struct passwd *passwd = getpwuid(s->st_uid);
 		struct group *group_ = getgrgid(s->st_gid);
@@ -544,10 +645,11 @@ setOwnerAndGroupAttributes(OFMutableFileAttributes attributes, Stat *s)
 			[attributes setObject: group
 				       forKey: OFFileGroupOwnerAccountName];
 		}
-# ifdef OF_HAVE_THREADS
+#  ifdef OF_HAVE_THREADS
 	} @finally {
 		[passwdMutex unlock];
 	}
+#  endif
 # endif
 #endif
 }
@@ -900,7 +1002,7 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 	return S_ISDIR(s.st_mode);
 }
 
-- (OFStream *)openItemAtIRI: (OFIRI *)IRI mode: (OFString *)mode
+- (OF_KINDOF(OFStream *))openItemAtIRI: (OFIRI *)IRI mode: (OFString *)mode
 {
 	void *pool = objc_autoreleasePoolPush();
 	OFFile *file;
@@ -966,6 +1068,27 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 	setExtendedAttributes(ret, IRI);
 #endif
 
+#ifdef OF_AMIGAOS
+	[ret setObject: [OFNumber numberWithUnsignedLong: s.protection]
+		forKey: OFFileAmigaProtection];
+
+	if (strlen(s.comment) > 0) {
+		OFStringEncoding encoding = [OFLocale encoding];
+
+		[ret setObject: [OFString stringWithCString: s.comment
+						   encoding: encoding]
+			forKey: OFFileAmigaComment];
+	}
+#endif
+
+#ifdef OF_DJGPP
+	int attrs = _chmod([path cStringWithEncoding: [OFLocale encoding]], 0);
+
+	if (attrs != -1)
+		[ret setObject: [OFNumber numberWithInt: attrs]
+			forKey: OFFileMSDOSAttributes];
+#endif
+
 	objc_autoreleasePoolPop(pool);
 
 	return ret;
@@ -1154,7 +1277,86 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 		   ofItemAtIRI: (OFIRI *)IRI
 		    attributes: (OFFileAttributes)attributes OF_DIRECT
 {
-#ifdef OF_FILE_MANAGER_SUPPORTS_PERMISSIONS
+#if defined(OF_AMIGAOS)
+	const char *path;
+	mode_t mode = (mode_t)permissions.unsignedLongValue;
+	BPTR lock;
+# ifdef OF_AMIGAOS4
+	struct ExamineData *ed;
+# else
+	struct FileInfoBlock fib;
+# endif
+	uint32_t protection;
+
+	if ([attributes objectForKey: OFFileAmigaProtection] != nil)
+		/* Amiga file protection trumps POSIX permissions */
+		return;
+
+	path = [IRI.fileSystemRepresentation
+	    cStringWithEncoding: [OFLocale encoding]];
+
+	if ((lock = Lock(path, SHARED_LOCK)) == 0)
+		@throw [OFSetItemAttributesFailedException
+		    exceptionWithIRI: IRI
+			  attributes: attributes
+		     failedAttribute: OFFilePOSIXPermissions
+			       errNo: lastError()];
+
+# if defined(OF_MORPHOS)
+	if (!Examine64(lock, &fib, TAG_END)) {
+# elif defined(OF_AMIGAOS4)
+	if ((ed = ExamineObjectTags(EX_FileLockInput, lock, TAG_END)) == NULL) {
+# else
+	if (!Examine(lock, &fib)) {
+# endif
+		int error = lastError();
+		UnLock(lock);
+		@throw [OFSetItemAttributesFailedException
+		    exceptionWithIRI: IRI
+			  attributes: attributes
+		     failedAttribute: OFFilePOSIXPermissions
+			       errNo: error];
+	}
+
+	UnLock(lock);
+
+# ifdef OF_AMIGAOS4
+	protection = ed->Protection;
+# else
+	protection = fib.fib_Protection;
+# endif
+
+	/* Remove the bits that can be derived from POSIX permissions */
+	protection &= ~(FIBF_READ | FIBF_WRITE | FIBF_EXECUTE |
+	    FIBF_GRP_READ | FIBF_GRP_WRITE | FIBF_GRP_EXECUTE |
+	    FIBF_OTR_READ | FIBF_OTR_WRITE | FIBF_OTR_EXECUTE);
+
+	if (!(mode & 0400))
+		protection |= FIBF_READ;
+	if (!(mode & 0200))
+		protection |= FIBF_WRITE;
+	if (!(mode & 0100))
+		protection |= FIBF_EXECUTE;
+	if (mode & 0040)
+		protection |= FIBF_GRP_READ;
+	if (mode & 0020)
+		protection |= FIBF_GRP_WRITE;
+	if (mode & 0010)
+		protection |= FIBF_GRP_EXECUTE;
+	if (mode & 0004)
+		protection |= FIBF_OTR_READ;
+	if (mode & 0002)
+		protection |= FIBF_OTR_WRITE;
+	if (mode & 0001)
+		protection |= FIBF_OTR_EXECUTE;
+
+	if (!SetProtection(path, protection))
+		@throw [OFSetItemAttributesFailedException
+		    exceptionWithIRI: IRI
+			  attributes: attributes
+		     failedAttribute: OFFileAmigaProtection
+			       errNo: lastError()];
+#elif defined(OF_FILE_MANAGER_SUPPORTS_PERMISSIONS)
 	mode_t mode = (mode_t)permissions.unsignedLongValue;
 	OFString *path = IRI.fileSystemRepresentation;
 	int status;
@@ -1178,51 +1380,175 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 #endif
 }
 
-- (void)of_setOwnerAccountName: (OFString *)owner
-      andGroupOwnerAccountName: (OFString *)group
+- (void)of_setOwnerAccountName: (OFString *)ownerAccountName
+      andGroupOwnerAccountName: (OFString *)groupOwnerAccountName
+		ownerAccountID: (OFNumber *)ownerAccountID
+	   groupOwnerAccountID: (OFNumber *)groupOwnerAccountID
 		   ofItemAtIRI: (OFIRI *)IRI
-		  attributeKey: (OFFileAttributeKey)attributeKey
 		    attributes: (OFFileAttributes)attributes OF_DIRECT
 {
-#ifdef OF_FILE_MANAGER_SUPPORTS_OWNER
+#if defined(OF_AMIGAOS)
+	const char *path;
+# ifndef OF_AMIGAOS4
+	BPTR lock;
+	struct FileInfoBlock fib;
+# endif
+	uint32_t uid = 0;
+	uint32_t gid = 0;
+
+	if (ownerAccountName != nil || groupOwnerAccountName != nil)
+		@throw [OFNotImplementedException
+		    exceptionWithSelector: @selector(setAttributes:ofItemAtIRI:)
+				   object: self];
+
+	if (ownerAccountID == nil && groupOwnerAccountID == nil)
+		@throw [OFInvalidArgumentException exception];
+
+	path = [IRI.fileSystemRepresentation
+	    cStringWithEncoding: [OFLocale encoding]];
+
+	if (ownerAccountID == nil || groupOwnerAccountID == nil) {
+# ifdef OF_AMIGAOS4
+		if (!GetOwnerInfoTags(OI_StringNameInput, path,
+		    OI_OwnerUID, &uid, OI_OwnerGID, &gid, TAG_END)) {
+			OFFileAttributeKey key = (ownerAccountID != nil
+			    ? OFFileOwnerAccountID : OFFileGroupOwnerAccountID);
+
+			@throw [OFSetItemAttributesFailedException
+			    exceptionWithIRI: IRI
+				  attributes: attributes
+			     failedAttribute: key
+				       errNo: lastError()];
+		}
+# else
+		if ((lock = Lock(path, SHARED_LOCK)) == 0) {
+			OFFileAttributeKey key = (ownerAccountID != nil
+			    ? OFFileOwnerAccountID : OFFileGroupOwnerAccountID);
+
+			@throw [OFSetItemAttributesFailedException
+			    exceptionWithIRI: IRI
+				  attributes: attributes
+			     failedAttribute: key
+				       errNo: lastError()];
+		}
+
+		if (!Examine(lock, &fib)) {
+			OFFileAttributeKey key = (ownerAccountID != nil
+			    ? OFFileOwnerAccountID : OFFileGroupOwnerAccountID);
+			int error = lastError();
+			UnLock(lock);
+			@throw [OFSetItemAttributesFailedException
+			    exceptionWithIRI: IRI
+				  attributes: attributes
+			     failedAttribute: key
+				       errNo: error];
+		}
+
+		UnLock(lock);
+
+		uid = fib.fib_OwnerUID;
+		gid = fib.fib_OwnerGID;
+# endif
+	}
+
+	if (ownerAccountID != nil) {
+		unsigned long tmp = ownerAccountID.unsignedLongValue;
+		if (tmp > UINT32_MAX)
+			@throw [OFOutOfRangeException exception];
+		uid = (uint32_t)tmp;
+	}
+
+	if (groupOwnerAccountID != nil) {
+		unsigned long tmp = groupOwnerAccountID.unsignedLongValue;
+		if (tmp > UINT32_MAX)
+			@throw [OFOutOfRangeException exception];
+		gid = (uint32_t)tmp;
+	}
+
+# ifdef OF_AMIGAOS4
+	if (!SetOwnerInfoTags(OI_StringNameInput, path, OI_OwnerUID, uid,
+	    OI_OwnerGID, gid, TAG_END)) {
+# else
+	if (uid > UINT16_MAX || gid > UINT16_MAX)
+		@throw [OFOutOfRangeException exception];
+
+	if (!SetOwner(path, uid << 16 | gid)) {
+# endif
+		OFFileAttributeKey key = (ownerAccountID != nil
+		    ? OFFileOwnerAccountID : OFFileGroupOwnerAccountID);
+
+		@throw [OFSetItemAttributesFailedException
+		    exceptionWithIRI: IRI
+			  attributes: attributes
+		     failedAttribute: key
+			       errNo: lastError()];
+	}
+#elif defined(OF_FILE_MANAGER_SUPPORTS_OWNER)
 	OFString *path = IRI.fileSystemRepresentation;
 	uid_t uid = -1;
 	gid_t gid = -1;
 	OFStringEncoding encoding;
 
-	if (owner == nil && group == nil)
+	if (ownerAccountName == nil && groupOwnerAccountName == nil &&
+	    ownerAccountID == nil && groupOwnerAccountID == nil)
 		@throw [OFInvalidArgumentException exception];
 
 	encoding = [OFLocale encoding];
 
+	if (ownerAccountID != nil) {
+		unsigned long tmp = ownerAccountID.unsignedLongValue;
+
+		if (tmp > UID_MAX)
+			@throw [OFOutOfRangeException exception];
+
+		uid = (uid_t)tmp;
+	}
+
+	if (groupOwnerAccountID != nil) {
+		unsigned long tmp = groupOwnerAccountID.unsignedLongValue;
+
+		if (tmp > GID_MAX)
+			@throw [OFOutOfRangeException exception];
+
+		uid = (gid_t)tmp;
+	}
+
 # ifdef OF_HAVE_THREADS
 	[passwdMutex lock];
 	@try {
 # endif
-		if (owner != nil) {
+		if (ownerAccountName != nil) {
 			struct passwd *passwd;
 
-			if ((passwd = getpwnam([owner
+			errno = 0;
+
+			if ((passwd = getpwnam([ownerAccountName
 			    cStringWithEncoding: encoding])) == NULL)
 				@throw [OFSetItemAttributesFailedException
 				    exceptionWithIRI: IRI
 					  attributes: attributes
-				     failedAttribute: attributeKey
+				     failedAttribute: OFFileOwnerAccountName
 					       errNo: errno];
 
 			uid = passwd->pw_uid;
 		}
 
-		if (group != nil) {
+		if (groupOwnerAccountName != nil) {
 			struct group *group_;
 
-			if ((group_ = getgrnam([group
-			    cStringWithEncoding: encoding])) == NULL)
+			errno = 0;
+
+			if ((group_ = getgrnam([groupOwnerAccountName
+			    cStringWithEncoding: encoding])) == NULL) {
+				OFFileAttributeKey key =
+				    OFFileGroupOwnerAccountName;
+
 				@throw [OFSetItemAttributesFailedException
 				    exceptionWithIRI: IRI
 					  attributes: attributes
-				     failedAttribute: attributeKey
+				     failedAttribute: key
 					       errNo: errno];
+			}
 
 			gid = group_->gr_gid;
 		}
@@ -1232,17 +1558,82 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 	}
 # endif
 
-	if (chown([path cStringWithEncoding: encoding], uid, gid) != 0)
+	if (chown([path cStringWithEncoding: encoding], uid, gid) != 0) {
+		OFFileAttributeKey key;
+
+		if (ownerAccountName != nil)
+			key = OFFileOwnerAccountName;
+		else if (groupOwnerAccountName != nil)
+			key = OFFileGroupOwnerAccountName;
+		else if (ownerAccountID != nil)
+			key = OFFileOwnerAccountID;
+		else if (groupOwnerAccountID != nil)
+			key = OFFileGroupOwnerAccountID;
+		else
+			OFAssert(0);
+
 		@throw [OFSetItemAttributesFailedException
 		    exceptionWithIRI: IRI
 			  attributes: attributes
-		     failedAttribute: attributeKey
+		     failedAttribute: key
 			       errNo: errno];
+	}
 #else
 	OF_UNRECOGNIZED_SELECTOR
 #endif
 }
 
+#ifdef OF_AMIGAOS
+- (void)of_setAmigaProtection: (OFNumber *)protection
+		  ofItemAtIRI: (OFIRI *)IRI
+		   attributes: (OFFileAttributes)attributes OF_DIRECT
+{
+	OFString *path = IRI.fileSystemRepresentation;
+
+	if (!SetProtection([path cStringWithEncoding: [OFLocale encoding]],
+	    protection.intValue))
+		@throw [OFSetItemAttributesFailedException
+		    exceptionWithIRI: IRI
+			  attributes: attributes
+		     failedAttribute: OFFileAmigaProtection
+			       errNo: lastError()];
+}
+
+- (void)of_setAmigaComment: (OFString *)comment
+	       ofItemAtIRI: (OFIRI *)IRI
+		attributes: (OFFileAttributes)attributes OF_DIRECT
+{
+	OFString *path = IRI.fileSystemRepresentation;
+	OFStringEncoding encoding = [OFLocale encoding];
+
+	if (!SetComment([path cStringWithEncoding: encoding],
+	    [comment cStringWithEncoding: encoding]))
+		@throw [OFSetItemAttributesFailedException
+		    exceptionWithIRI: IRI
+			  attributes: attributes
+		     failedAttribute: OFFileAmigaComment
+			       errNo: 0];
+}
+#endif
+
+#ifdef OF_DJGPP
+- (void)of_setMSDOSAttributes: (OFNumber *)MSDOSAttributes
+		  ofItemAtIRI: (OFIRI *)IRI
+		   attributes: (OFFileAttributes)attributes OF_DIRECT
+{
+	OFString *path = IRI.fileSystemRepresentation;
+
+	if (_chmod([path cStringWithEncoding: [OFLocale encoding]], 1,
+	    MSDOSAttributes.intValue & ~(OFFileMSDOSAttributeVolumeLabel |
+	    OFFileMSDOSAttributeDirectory)) == -1)
+		@throw [OFSetItemAttributesFailedException
+		    exceptionWithIRI: IRI
+			  attributes: attributes
+		     failedAttribute: OFFileMSDOSAttributes
+			       errNo: errno];
+}
+#endif
+
 - (void)setAttributes: (OFFileAttributes)attributes ofItemAtIRI: (OFIRI *)IRI
 {
 	void *pool = objc_autoreleasePoolPush();
@@ -1250,6 +1641,8 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 	OFEnumerator *objectEnumerator;
 	OFFileAttributeKey key;
 	id object;
+	OFString *ownerAccountName, *groupOwnerAccountName;
+	OFNumber *ownerAccountID, *groupOwnerAccountID;
 	OFDate *lastAccessDate, *modificationDate;
 
 	if (IRI == nil)
@@ -1263,31 +1656,55 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 
 	while ((key = [keyEnumerator nextObject]) != nil &&
 	    (object = [objectEnumerator nextObject]) != nil) {
-		if ([key isEqual: OFFileModificationDate] ||
+		if ([key isEqual: OFFileOwnerAccountName] ||
+		    [key isEqual: OFFileOwnerAccountID] ||
+		    [key isEqual: OFFileGroupOwnerAccountName] ||
+		    [key isEqual: OFFileGroupOwnerAccountID] ||
+		    [key isEqual: OFFileModificationDate] ||
 		    [key isEqual: OFFileLastAccessDate])
 			continue;
 		else if ([key isEqual: OFFilePOSIXPermissions])
 			[self of_setPOSIXPermissions: object
 					 ofItemAtIRI: IRI
 					  attributes: attributes];
-		else if ([key isEqual: OFFileOwnerAccountName])
-			[self of_setOwnerAccountName: object
-			    andGroupOwnerAccountName: nil
-					 ofItemAtIRI: IRI
-					attributeKey: key
-					  attributes: attributes];
-		else if ([key isEqual: OFFileGroupOwnerAccountName])
-			[self of_setOwnerAccountName: nil
-			    andGroupOwnerAccountName: object
-					 ofItemAtIRI: IRI
-					attributeKey: key
-					  attributes: attributes];
+#ifdef OF_AMIGAOS
+		else if ([key isEqual: OFFileAmigaProtection])
+			[self of_setAmigaProtection: object
+					ofItemAtIRI: IRI
+					 attributes: attributes];
+		else if ([key isEqual: OFFileAmigaComment])
+			[self of_setAmigaComment: object
+				     ofItemAtIRI: IRI
+				      attributes: attributes];
+#endif
+#ifdef OF_DJGPP
+		else if ([key isEqual: OFFileMSDOSAttributes])
+			[self of_setMSDOSAttributes: object
+					ofItemAtIRI: IRI
+					 attributes: attributes];
+#endif
 		else
 			@throw [OFNotImplementedException
 			    exceptionWithSelector: _cmd
 					   object: self];
 	}
 
+	ownerAccountName = [attributes objectForKey: OFFileOwnerAccountName];
+	groupOwnerAccountName = [attributes
+	    objectForKey: OFFileGroupOwnerAccountName];
+	ownerAccountID = [attributes objectForKey: OFFileOwnerAccountID];
+	groupOwnerAccountID = [attributes
+	    objectForKey: OFFileGroupOwnerAccountID];
+
+	if (ownerAccountName != nil || groupOwnerAccountName != nil ||
+	    ownerAccountID != nil || groupOwnerAccountID != nil)
+		[self of_setOwnerAccountName: ownerAccountName
+		    andGroupOwnerAccountName: groupOwnerAccountName
+			      ownerAccountID: ownerAccountID
+			 groupOwnerAccountID: groupOwnerAccountID
+				 ofItemAtIRI: IRI
+				  attributes: attributes];
+
 	lastAccessDate = [attributes objectForKey: OFFileLastAccessDate];
 	modificationDate = [attributes objectForKey: OFFileModificationDate];
 
