@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -25,18 +25,18 @@ OF_ASSUME_NONNULL_BEGIN
 /** @file */
 
 #ifdef OF_HAVE_FILES
-# if (defined(OF_HAVE_CHMOD) && !defined(OF_AMIGAOS)) || defined(DOXYGEN)
+# if (defined(OF_HAVE_CHMOD) && !defined(OF_NINTENDO_DS)) || defined(DOXYGEN)
 #  define OF_FILE_MANAGER_SUPPORTS_PERMISSIONS
 # endif
-# if (defined(OF_HAVE_CHOWN) && !defined(OF_AMIGAOS)) || defined(DOXYGEN)
+# if defined(OF_HAVE_CHOWN) || defined(DOXYGEN)
 #  define OF_FILE_MANAGER_SUPPORTS_OWNER
 # endif
-# if (defined(OF_HAVE_LINK) && !defined(OF_AMIGAOS) && !defined(OF_HAIKU)) || \
-    defined(OF_WINDOWS) || defined(DOXYGEN)
+# if (defined(OF_HAVE_LINK) && !defined(OF_AMIGAOS) && \
+    !defined(OF_NINTENDO_DS)) || defined(OF_WINDOWS) || defined(DOXYGEN)
 #  define OF_FILE_MANAGER_SUPPORTS_LINKS
 # endif
-# if (defined(OF_HAVE_SYMLINK) && !defined(OF_AMIGAOS)) || \
-    defined(OF_WINDOWS) || defined(DOXYGEN)
+# if (defined(OF_HAVE_SYMLINK) && !defined(OF_AMIGAOS) && \
+    !defined(OF_NINTENDO_DS)) || defined(OF_WINDOWS) || defined(DOXYGEN)
 #  define OF_FILE_MANAGER_SUPPORTS_SYMLINKS
 # endif
 # if defined(OF_LINUX) || defined(OF_MACOS) || defined(OF_FREEBSD) || \
@@ -50,8 +50,65 @@ OF_ASSUME_NONNULL_BEGIN
 @class OFConstantString;
 @class OFDate;
 @class OFIRI;
+@class OFNumber;
 @class OFString;
 
+/**
+ * @brief Amiga file protection. This is a bit mask and the values are ORed.
+ */
+typedef enum {
+	/** @brief The file cannot be deleted. */
+	OFFileAmigaDeleteProtected  = 0x0001,
+	/** @brief The file cannot be executed. */
+	OFFileAmigaExecuteProtected = 0x0002,
+	/** @brief The file cannot be written to. */
+	OFFileAmigaWriteProtected   = 0x0004,
+	/** @brief The file cannot be read. */
+	OFFileAmigaReadProtected    = 0x0008,
+	/** @brief The file has not been changed since it was archived. */
+	OFFileAmigaArchived         = 0x0010,
+	/** @brief The file is a reentrant program and can be made resident. */
+	OFFileAmigaPure             = 0x0020,
+	/** @brief The file is a script. */
+	OFFileAmigaScript           = 0x0040,
+	/** @brief The file is made resident on first execution. */
+	OFFileAmigaHold             = 0x0080,
+	/** @brief The file can be deleted by the group. */
+	OFFileAmigaGroupDeletable   = 0x0100,
+	/** @brief The file can be executed by the group. */
+	OFFileAmigaGroupExecutable  = 0x0200,
+	/** @brief The file can be written to by the group. */
+	OFFileAmigaGroupWritable    = 0x0400,
+	/** @brief The file can be read by group. */
+	OFFileAmigaGroupReadable    = 0x0800,
+	/** @brief The file can be deleted by others. */
+	OFFileAmigaOtherDeletable   = 0x1000,
+	/** @brief The file can be executed by others. */
+	OFFileAmigaOtherExecutable  = 0x2000,
+	/** @brief The file can be written to by others. */
+	OFFileAmigaOtherWritable    = 0x4000,
+	/** @brief The file can be read by others. */
+	OFFileAmigaOtherReadable    = 0x8000
+} OFFileAmigaProtectionMask;
+
+/**
+ * @brief MS-DOS attributes. This is a bit mask and the values are ORed.
+ */
+typedef enum {
+	/** @brief The file is read-only. */
+	OFFileMSDOSAttributeReadOnly    = 0x01,
+	/** @brief The file is hidden. */
+	OFFileMSDOSAttributeHidden      = 0x02,
+	/** @brief The file is a system file. */
+	OFFileMSDOSAttributeSystem      = 0x04,
+	/** @brief The file is a volume label. */
+	OFFileMSDOSAttributeVolumeLabel = 0x08,
+	/** @brief The file is a directory. */
+	OFFileMSDOSAttributeDirectory   = 0x10,
+	/** @brief The file should be archived. */
+	OFFileMSDOSAttributeArchive     = 0x20,
+} OFFileMSDOSAttributesMask;
+
 /**
  * @brief A key for a file attribute in the file attributes dictionary.
  *
@@ -70,6 +127,9 @@ OF_ASSUME_NONNULL_BEGIN
  *  * @ref OFFileCreationDate
  *  * @ref OFFileSymbolicLinkDestination
  *  * @ref OFFileExtendedAttributesNames
+ *  * @ref OFFileAmigaProtection
+ *  * @ref OFFileAmigaComment
+ *  * @ref OFFileMSDOSAttributes
  *
  * Other IRI schemes might not have all keys and might have keys not listed.
  */
@@ -216,6 +276,25 @@ extern const OFFileAttributeKey OFFileSymbolicLinkDestination;
  */
 extern const OFFileAttributeKey OFFileExtendedAttributesNames;
 
+/**
+ * @brief The Amiga file protection as an @ref OFNumber.
+ *
+ * See @ref OFFileAmigaProtectionMask for possible values.
+ */
+extern const OFFileAttributeKey OFFileAmigaProtection;
+
+/**
+ * @brief The Amiga comment as an @ref OFString.
+ */
+extern const OFFileAttributeKey OFFileAmigaComment;
+
+/**
+ * @brief The MS-DOS attributes as an @ref OFNumber.
+ *
+ * See @ref OFFileMSDOSAttributesMask for possible values.
+ */
+extern const OFFileAttributeKey OFFileMSDOSAttributes;
+
 /**
  * @brief A regular file.
  */
@@ -1001,6 +1080,27 @@ OF_SUBCLASSING_RESTRICTED
  */
 @property (readonly, nonatomic)
     OFArray OF_GENERIC(OFString *) *fileExtendedAttributesNames;
+
+/**
+ * @brief The @ref OFFileAmigaProtection key from the dictionary.
+ *
+ * @throw OFUndefinedKeyException The key is missing
+ */
+@property (readonly, nonatomic) OFFileAmigaProtectionMask fileAmigaProtection;
+
+/**
+ * @brief The @ref OFFileAmigaComment key from the dictionary.
+ *
+ * @throw OFUndefinedKeyException The key is missing
+ */
+@property (readonly, nonatomic) OFString *fileAmigaComment;
+
+/**
+ * @brief The @ref OFFileMSDOSAttributes key from the dictionary.
+ *
+ * @throw OFUndefinedKeyException The key is missing
+ */
+@property (readonly, nonatomic) OFFileMSDOSAttributesMask fileMSDOSAttributes;
 @end
 
 OF_ASSUME_NONNULL_END
