@@ -1175,4 +1175,21 @@ OF_SINGLETON_METHODS
 {
 	return attributeForKeyOrException(self, OFFileExtendedAttributesNames);
 }
+
+- (OFFileAmigaProtectionMask)fileAmigaProtection
+{
+	return [attributeForKeyOrException(self,
+	    OFFileAmigaProtection) intValue];
+}
+
+- (OFString *)fileAmigaComment
+{
+	return attributeForKeyOrException(self, OFFileAmigaComment);
+}
+
+- (OFFileMSDOSAttributesMask)fileMSDOSAttributes
+{
+	return [attributeForKeyOrException(self,
+	    OFFileMSDOSAttributes) intValue];
+}
 @end
