@@ -99,7 +99,7 @@
 @end
 
 @implementation OFHTTPIRIHandler
-- (OFStream *)openItemAtIRI: (OFIRI *)IRI mode: (OFString *)mode
+- (OF_KINDOF(OFStream *))openItemAtIRI: (OFIRI *)IRI mode: (OFString *)mode
 {
 	void *pool = objc_autoreleasePoolPush();
 	OFHTTPClient *client = [OFHTTPClient client];
