@@ -17,5 +17,9 @@
  * <https://www.gnu.org/licenses/>.
  */
 
-#define OF_INFLATE64_STREAM_M
-#include "OFInflateStream.m"
+#include "config.h"
+
+#import "OFInflate64Stream.h"
+
+@implementation OFInflate64Stream
+@end
