@@ -57,6 +57,12 @@ parseMSDOSDate(uint32_t MSDOSDate)
 }
 
 @implementation OFLHAArchiveEntry
+/*
+ * The following are optional in OFArchiveEntry, but Apple GCC 4.0.1 is buggy
+ * and needs this to stop complaining.
+ */
+@dynamic targetFileName, deviceMajor, deviceMinor;
+
 static void
 parseFileNameExtension(OFLHAArchiveEntry *entry, OFData *extension,
     OFStringEncoding encoding)
@@ -221,6 +227,36 @@ parseFileSizeExtension(OFLHAArchiveEntry *entry, OFData *extension,
 	entry->_uncompressedSize = OFFromLittleEndian64(tmp);
 }
 
+static void
+parseMSDOSAttributesExtension(OFLHAArchiveEntry *entry, OFData *extension,
+    OFStringEncoding encoding)
+{
+	uint16_t tmp;
+
+	if (extension.count != 3)
+		@throw [OFInvalidFormatException exception];
+
+	objc_release(entry->_MSDOSAttributes);
+	entry->_MSDOSAttributes = nil;
+
+	memcpy(&tmp, (char *)extension.items + 1, 2);
+	entry->_MSDOSAttributes = [OFNumber numberWithUnsignedShort:
+	    OFFromLittleEndian16(tmp)];
+}
+
+static void
+parseAmigaCommentExtension(OFLHAArchiveEntry *entry, OFData *extension,
+    OFStringEncoding encoding)
+{
+	objc_release(entry->_amigaComment);
+	entry->_amigaComment = nil;
+
+	entry->_amigaComment = [[OFString alloc]
+	    initWithCString: (char *)extension.items + 1
+		   encoding: encoding
+		     length: [extension count] - 1];
+}
+
 static bool
 parseExtension(OFLHAArchiveEntry *entry, OFData *extension,
     OFStringEncoding encoding, bool allowFileName)
@@ -239,6 +275,9 @@ parseExtension(OFLHAArchiveEntry *entry, OFData *extension,
 	case 0x3F:
 		function = parseCommentExtension;
 		break;
+	case 0x40:
+		function = parseMSDOSAttributesExtension;
+		break;
 	case 0x42:
 		function = parseFileSizeExtension;
 		break;
@@ -257,6 +296,9 @@ parseExtension(OFLHAArchiveEntry *entry, OFData *extension,
 	case 0x54:
 		function = parseModificationDateExtension;
 		break;
+	case 0x71:
+		function = parseAmigaCommentExtension;
+		break;
 	}
 
 	if (function == NULL)
@@ -392,8 +434,9 @@ getFileNameAndDirectoryName(OFLHAArchiveEntry *entry, OFStringEncoding encoding,
 		case 1:;
 			void *pool = objc_autoreleasePoolPush();
 			uint8_t extendedAreaSize;
+			char fileNameBuffer[255];
 			uint8_t fileNameLength;
-			OFString *fileName;
+			char *amigaCommentPtr;
 
 			if (header[0] < (21 - 2) + 1 + 2)
 				@throw [OFInvalidFormatException exception];
@@ -401,12 +444,11 @@ getFileNameAndDirectoryName(OFLHAArchiveEntry *entry, OFStringEncoding encoding,
 			_modificationDate = objc_retain(parseMSDOSDate(date));
 
 			fileNameLength = [stream readInt8];
-			fileName = [stream readStringWithLength: fileNameLength
-						       encoding: encoding];
-			fileName = [fileName
-			    stringByReplacingOccurrencesOfString: @"\\"
-						      withString: @"/"];
-			_fileName = [fileName copy];
+			[stream readIntoBuffer: fileNameBuffer
+				   exactLength: fileNameLength];
+
+			_MSDOSAttributes = [[OFNumber alloc]
+			    initWithUnsignedChar: header[19]];
 
 			_CRC16 = [stream readLittleEndianInt16];
 
@@ -428,6 +470,31 @@ getFileNameAndDirectoryName(OFLHAArchiveEntry *entry, OFStringEncoding encoding,
 				extendedAreaSize -= 1 + 2;
 			}
 
+			amigaCommentPtr = memchr(fileNameBuffer, '\0',
+			    fileNameLength);
+			if (amigaCommentPtr != NULL) {
+				size_t newFileNameLength =
+				    (amigaCommentPtr - fileNameBuffer);
+
+				_amigaComment = [[OFString alloc]
+				    initWithCString: amigaCommentPtr + 1
+					   encoding: encoding
+					     length: fileNameLength -
+						     newFileNameLength - 1];
+
+				fileNameLength = newFileNameLength;
+			}
+
+			for (size_t i = 0; i < fileNameLength; i++)
+				if (fileNameBuffer[i] == '\xFF' ||
+				    fileNameBuffer[i] == '\\')
+					fileNameBuffer[i] = '/';
+
+			_fileName = [[OFString alloc]
+			    initWithCString: fileNameBuffer
+				   encoding: encoding
+				     length: fileNameLength];
+
 			/* Skip extended area */
 			if ([stream isKindOfClass: [OFSeekableStream class]])
 				[(OFSeekableStream *)stream
@@ -457,6 +524,21 @@ getFileNameAndDirectoryName(OFLHAArchiveEntry *entry, OFStringEncoding encoding,
 			_CRC16 = [stream readLittleEndianInt16];
 			_operatingSystemIdentifier = [stream readInt8];
 
+			if (_operatingSystemIdentifier == 'A') {
+				/*
+				 * Amiga LHA uses seconds since 1970-01-01 in
+				 * local time rather than UTC.
+				 */
+				OFString *tmpStr = [_modificationDate
+				    dateStringWithFormat: @"%Y-%m-%d %H:%M:%S"];
+				OFDate *tmpDate = [OFDate
+				    dateWithLocalDateString: tmpStr
+						     format: @"%Y-%m-%d "
+							     @"%H:%M:%S"];
+				objc_release(_modificationDate);
+				_modificationDate = objc_retain(tmpDate);
+			}
+
 			if (_headerLevel == 3)
 				/* Size of entire header */
 				padding = [stream readLittleEndianInt32];
@@ -528,6 +610,8 @@ getFileNameAndDirectoryName(OFLHAArchiveEntry *entry, OFStringEncoding encoding,
 	objc_release(_groupOwnerAccountID);
 	objc_release(_ownerAccountName);
 	objc_release(_groupOwnerAccountName);
+	objc_release(_MSDOSAttributes);
+	objc_release(_amigaComment);
 	objc_release(_extensions);
 
 	[super dealloc];
@@ -565,6 +649,8 @@ getFileNameAndDirectoryName(OFLHAArchiveEntry *entry, OFStringEncoding encoding,
 		copy->_ownerAccountName = [_ownerAccountName copy];
 		copy->_groupOwnerAccountName = [_groupOwnerAccountName copy];
 		copy->_extensions = [_extensions copy];
+		copy->_MSDOSAttributes = objc_retain(_MSDOSAttributes);
+		copy->_amigaComment = [_amigaComment copy];
 	} @catch (id e) {
 		objc_release(copy);
 		@throw e;
@@ -581,6 +667,14 @@ getFileNameAndDirectoryName(OFLHAArchiveEntry *entry, OFStringEncoding encoding,
 	return [_directoryName stringByAppendingString: _fileName];
 }
 
+- (OFArchiveEntryFileType)fileType
+{
+	if ([_fileName hasSuffix: @"/"])
+		return OFArchiveEntryFileTypeDirectory;
+
+	return OFArchiveEntryFileTypeRegular;
+}
+
 - (OFString *)compressionMethod
 {
 	return _compressionMethod;
@@ -646,6 +740,16 @@ getFileNameAndDirectoryName(OFLHAArchiveEntry *entry, OFStringEncoding encoding,
 	return _groupOwnerAccountName;
 }
 
+- (OFNumber *)MSDOSAttributes
+{
+	return _MSDOSAttributes;
+}
+
+- (OFString *)amigaComment
+{
+	return _amigaComment;
+}
+
 - (OFArray OF_GENERIC(OFData *) *)extensions
 {
 	return _extensions;
@@ -688,8 +792,21 @@ getFileNameAndDirectoryName(OFLHAArchiveEntry *entry, OFStringEncoding encoding,
 	tmp32 = OFToLittleEndian32((uint32_t)_uncompressedSize);
 	[data addItems: &tmp32 count: sizeof(tmp32)];
 
-	tmp32 = OFToLittleEndian32(
-	    (uint32_t)_modificationDate.timeIntervalSince1970);
+	if (_operatingSystemIdentifier == 'A') {
+		/*
+		 * Amiga LHA uses seconds since 1970-01-01 in local time rather
+		 * than UTC.
+		 */
+		OFString *tmpStr = [_modificationDate
+		    localDateStringWithFormat: @"%Y-%m-%d %H:%M:%S"];
+		OFDate *tmpDate = [OFDate
+		    dateWithDateString: tmpStr
+				format: @"%Y-%m-%d %H:%M:%S"];
+		tmp32 = OFToLittleEndian32(
+		    (uint32_t)tmpDate.timeIntervalSince1970);
+	} else
+		tmp32 = OFToLittleEndian32(
+		    (uint32_t)_modificationDate.timeIntervalSince1970);
 	[data addItems: &tmp32 count: sizeof(tmp32)];
 
 	/* Reserved */
@@ -703,7 +820,10 @@ getFileNameAndDirectoryName(OFLHAArchiveEntry *entry, OFStringEncoding encoding,
 	[data addItems: &tmp16 count: sizeof(tmp16)];
 
 	/* Operating system identifier */
-	[data addItem: "U"];
+	if (_operatingSystemIdentifier != 0)
+		[data addItem: &_operatingSystemIdentifier];
+	else
+		[data addItem: "U"];
 
 	/* Common header. Contains CRC16, which is written at the end. */
 	tmp16 = OFToLittleEndian16(5);
@@ -751,6 +871,16 @@ getFileNameAndDirectoryName(OFLHAArchiveEntry *entry, OFStringEncoding encoding,
 	tmp64 = OFToLittleEndian64(_uncompressedSize);
 	[data addItems: &tmp64 count: sizeof(tmp64)];
 
+	if (_MSDOSAttributes != nil) {
+		tmp16 = OFToLittleEndian16(5);
+		[data addItems: &tmp16 count: sizeof(tmp16)];
+		[data addItem: "\x40"];
+
+		tmp16 =
+		    OFToLittleEndian16(_MSDOSAttributes.unsignedShortValue);
+		[data addItems: &tmp16 count: sizeof(tmp16)];
+	}
+
 	if (_POSIXPermissions != nil) {
 		tmp16 = OFToLittleEndian16(5);
 		[data addItems: &tmp16 count: sizeof(tmp16)];
@@ -807,6 +937,20 @@ getFileNameAndDirectoryName(OFLHAArchiveEntry *entry, OFStringEncoding encoding,
 			 count: length];
 	}
 
+	if (_amigaComment != nil) {
+		size_t length =
+		    [_amigaComment cStringLengthWithEncoding: encoding];
+
+		if (length > UINT16_MAX - 3)
+			@throw [OFOutOfRangeException exception];
+
+		tmp16 = OFToLittleEndian16((uint16_t)length + 3);
+		[data addItems: &tmp16 count: sizeof(tmp16)];
+		[data addItem: "\x71"];
+		[data addItems: [_amigaComment cStringWithEncoding: encoding]
+			 count: length];
+	}
+
 	for (OFData *extension in _extensions) {
 		size_t extensionLength = extension.count;
 
@@ -878,14 +1022,17 @@ getFileNameAndDirectoryName(OFLHAArchiveEntry *entry, OFStringEncoding encoding,
 	    @"\tOwner account ID = %@\n"
 	    @"\tGroup owner account ID = %@\n"
 	    @"\tOwner account name = %@\n"
-	    @"\tGroup owner accounut name = %@\n"
+	    @"\tGroup owner account name = %@\n"
+	    @"\tMS-DOS attributes = %@\n"
+	    @"\tAmiga comment = %@\n"
 	    @"\tExtensions: %@"
 	    @">",
 	    self.class, self.fileName, _compressionMethod, _compressedSize,
 	    _uncompressedSize, _modificationDate, _headerLevel, _CRC16,
 	    _operatingSystemIdentifier, _fileComment, POSIXPermissions,
 	    _ownerAccountID, _groupOwnerAccountID, _ownerAccountName,
-	    _groupOwnerAccountName, extensions];
+	    _groupOwnerAccountName, _MSDOSAttributes, _amigaComment,
+	    extensions];
 
 	objc_retain(ret);
 
