@@ -528,10 +528,14 @@ evaluateArray(OFArray *array, OFDictionary *variables)
 
 		if (OFStdIn.hasTerminal)
 			OFStdIn.encoding = _encoding;
-		if (OFStdOut.hasTerminal)
+		if (OFStdOut.hasTerminal) {
 			OFStdOut.encoding = _encoding;
-		if (OFStdErr.hasTerminal)
+			OFStdOut.allowsLossyEncoding = true;
+		}
+		if (OFStdErr.hasTerminal) {
 			OFStdErr.encoding = _encoding;
+			OFStdErr.allowsLossyEncoding = true;
+		}
 	} @catch (id e) {
 		objc_release(self);
 		@throw e;
@@ -683,8 +687,16 @@ OF_SINGLETON_METHODS
 						  length: i - last];
 				OFString *value = [variables objectForKey: var];
 
-				if (value != nil)
-					[ret appendString: value.description];
+				if (value != nil) {
+					value = value.description;
+
+					if (![var hasPrefix: @"@"])
+#define SBRCC stringByReplacingControlCharacters
+						value = value.SBRCC;
+#undef SBRCC
+
+					[ret appendString: value];
+				}
 
 				last = i + 1;
 				state = 0;
