@@ -33,6 +33,11 @@ OF_ASSUME_NONNULL_BEGIN
  */
 @property (readwrite, copy, nonatomic) OFString *fileName;
 
+/**
+ * @brief The file type of the entry.
+ */
+@property (readwrite, nonatomic) OFArchiveEntryFileType fileType;
+
 /**
  * @brief The compressed size of the entry's file.
  */
@@ -84,6 +89,22 @@ OF_ASSUME_NONNULL_BEGIN
  */
 @property OF_NULLABLE_PROPERTY (readwrite, retain, nonatomic)
     OFString *groupOwnerAccountName;
+
+/**
+ * @brief The file name of the target (for a hard link or symbolic link).
+ */
+@property OF_NULLABLE_PROPERTY (readwrite, copy, nonatomic)
+    OFString *targetFileName;
+
+/**
+ * @brief The device major (if the file is a device).
+ */
+@property (readwrite, nonatomic) unsigned long deviceMajor;
+
+/**
+ * @brief The device major (if the file is a device).
+ */
+@property (readwrite, nonatomic) unsigned long deviceMinor;
 @end
 
 OF_ASSUME_NONNULL_END
