@@ -34,25 +34,37 @@ OF_ASSUME_NONNULL_BEGIN
 /**
  * @brief The type of the archive entry.
  *
- * See @ref OFTarArchiveEntryType.
+ * @deprecated Use @ref OFMutableArchiveEntry#fileType instead.
  */
-@property (readwrite, nonatomic) OFTarArchiveEntryType type;
+#if OF_GCC_VERSION >= 405
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
+@property (readwrite, nonatomic) OFTarArchiveEntryType type
+    OF_DEPRECATED(ObjFW, 1, 5, "Use -[OFMutableArchiveEntry fileType] instead");
+#if OF_GCC_VERSION >= 405
+# pragma GCC diagnostic pop
+#endif
 
 /**
- * @brief The file name of the target (for a hard link or symbolic link).
+ * @brief The PAX extended header of the Tar archive entry.
+ *
+ * This is a map of @ref OFString to @ref OFData.
  */
 @property OF_NULLABLE_PROPERTY (readwrite, copy, nonatomic)
-    OFString *targetFileName;
+    OFDictionary OF_GENERIC(OFString *, OFData *) *extendedHeader;
 
 /**
- * @brief The device major (if the file is a device).
+ * @brief The Amiga Protection of the file (see @ref OFFileAmigaProtectionMask).
  */
-@property (readwrite, nonatomic) unsigned long deviceMajor;
+@property OF_NULLABLE_PROPERTY (readwrite, retain, nonatomic)
+    OFNumber *amigaProtection;
 
 /**
- * @brief The device major (if the file is a device).
+ * @brief The Amiga comment of the file.
  */
-@property (readwrite, nonatomic) unsigned long deviceMinor;
+@property OF_NULLABLE_PROPERTY (readwrite, copy, nonatomic)
+    OFString *amigaComment;
 
 /**
  * @brief Creates a new OFMutableTarArchiveEntry with the specified file name.
