@@ -386,6 +386,40 @@ OFEqualRects(OFRect rect1, OFRect rect2)
 	return true;
 }
 
+/**
+ * @brief Returns the intersection of the two rectangles or a rectangle with x,
+ *	  y, width and height set to 0 if the two rectangles don't intersect.
+ *
+ * @param rect1 The first rectangle
+ * @param rect2 The second rectangle
+ * @return The intersection of both rectangles
+ */
+static OF_INLINE OFRect
+OFIntersectionRect(OFRect rect1, OFRect rect2)
+{
+	OFRect rect;
+
+	rect.origin.x = (rect1.origin.x >= rect2.origin.x
+	    ? rect1.origin.x : rect2.origin.x);
+	rect.origin.y = (rect1.origin.y >= rect2.origin.y
+	    ? rect1.origin.y : rect2.origin.y);
+	rect.size.width = (rect1.origin.x + rect1.size.width <
+	    rect2.origin.x + rect2.size.width
+	    ? rect1.origin.x + rect1.size.width
+	    : rect2.origin.x + rect2.size.width) - rect.origin.x;
+	rect.size.height = (rect1.origin.y + rect1.size.height <
+	    rect2.origin.y + rect2.size.height
+	    ? rect1.origin.y + rect1.size.height
+	    : rect2.origin.y + rect2.size.height) - rect.origin.y;
+
+	if (rect.size.width <= 0.0f || rect.size.height <= 0.0f) {
+		rect.origin.x = rect.origin.y = 0.0f;
+		rect.size.width = rect.size.height = 0.0f;
+	}
+
+	return rect;
+}
+
 /**
  * @struct OFVector3D OFObject.h ObjFW/ObjFW.h
  *
@@ -971,7 +1005,7 @@ OF_ROOT_CLASS
  *
  * @warning You cannot make use of other classes from inside this method! Only
  *	    the class itself, its superclasses and instances of the class or
- *	    its superclassses can be messaged!
+ *	    its superclasses can be messaged!
  */
 + (void)load;
 
@@ -1759,7 +1793,7 @@ extern id OFAllocObject(Class class_, size_t extraSize, size_t extraAlignment,
  * @brief This function is called when a method is not found.
  *
  * It can also be called intentionally to indicate that a method is not
- * implemetned, for example in an abstract method. However, instead of calling
+ * implemented, for example in an abstract method. However, instead of calling
  * OFMethodNotFound directly, it is preferred to do the following:
  *
  *     - (void)abstractMethod
