@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -26,7 +26,9 @@
 #endif
 
 #import "OFPollKernelEventObserver.h"
+#import "OFArray.h"
 #import "OFData.h"
+#import "OFNull.h"
 #import "OFSocket.h"
 #import "OFSocket+Private.h"
 
@@ -50,10 +52,10 @@
 		    sizeof(struct pollfd)];
 		[_FDs addItem: &p];
 
-		_maxFD = _cancelFD[0];
-		_FDToObject = OFAllocMemory((size_t)_maxFD + 1, sizeof(id));
+		_objects = [[OFMutableArray alloc]
+		    initWithObject: [OFNull null]];
 	} @catch (id e) {
-		[self release];
+		objc_release(self);
 		@throw e;
 	}
 
@@ -62,8 +64,8 @@
 
 - (void)dealloc
 {
-	[_FDs release];
-	OFFreeMemory(_FDToObject);
+	objc_release(_FDs);
+	objc_release(_objects);
 
 	[super dealloc];
 }
@@ -71,9 +73,9 @@
 static void
 addObject(OFPollKernelEventObserver *self, id object, int fd, short events)
 {
+	struct pollfd p = { fd, events, 0 };
 	struct pollfd *FDs;
 	size_t count;
-	bool found;
 
 	if (fd < 0)
 		@throw [OFObserveKernelEventsFailedException
@@ -82,28 +84,16 @@ addObject(OFPollKernelEventObserver *self, id object, int fd, short events)
 
 	FDs = self->_FDs.mutableItems;
 	count = self->_FDs.count;
-	found = false;
 
 	for (size_t i = 0; i < count; i++) {
 		if (FDs[i].fd == fd) {
 			FDs[i].events |= events;
-			found = true;
-			break;
+			return;
 		}
 	}
 
-	if (!found) {
-		struct pollfd p = { fd, events, 0 };
-
-		if (fd > self->_maxFD) {
-			self->_maxFD = fd;
-			self->_FDToObject = OFResizeMemory(self->_FDToObject,
-			    (size_t)self->_maxFD + 1, sizeof(id));
-		}
-
-		self->_FDToObject[fd] = object;
-		[self->_FDs addItem: &p];
-	}
+	[self->_FDs addItem: &p];
+	[self->_objects addObject: object];
 }
 
 static void
@@ -125,11 +115,8 @@ removeObject(OFPollKernelEventObserver *self, id object, int fd, short events)
 			FDs[i].events &= ~events;
 
 			if (FDs[i].events == 0) {
-				/*
-				 * TODO: Remove from and resize _FDToObject,
-				 *	 adjust _maxFD.
-				 */
 				[self->_FDs removeItemAtIndex: i];
+				[self->_objects removeObjectAtIndex: i];
 			}
 
 			break;
@@ -169,6 +156,7 @@ removeObject(OFPollKernelEventObserver *self, id object, int fd, short events)
 {
 	void *pool;
 	struct pollfd *FDs;
+	OFArray *objects;
 	size_t nFDs;
 
 	if ([self processReadBuffers])
@@ -176,9 +164,12 @@ removeObject(OFPollKernelEventObserver *self, id object, int fd, short events)
 
 	pool = objc_autoreleasePoolPush();
 
-	FDs = [[[_FDs mutableCopy] autorelease] mutableItems];
+	FDs = [objc_autorelease([_FDs mutableCopy]) mutableItems];
+	objects = objc_autorelease([_objects copy]);
 	nFDs = _FDs.count;
 
+	OFAssert(objects.count == nFDs);
+
 #ifdef OPEN_MAX
 	if (nFDs > OPEN_MAX)
 		@throw [OFOutOfRangeException exception];
@@ -195,8 +186,6 @@ removeObject(OFPollKernelEventObserver *self, id object, int fd, short events)
 	}
 
 	for (size_t i = 0; i < nFDs; i++) {
-		OFAssert(FDs[i].fd <= _maxFD);
-
 		if (FDs[i].revents & POLLIN) {
 			void *pool2;
 
@@ -219,7 +208,7 @@ removeObject(OFPollKernelEventObserver *self, id object, int fd, short events)
 			if ([_delegate respondsToSelector:
 			    @selector(objectIsReadyForReading:)])
 				[_delegate objectIsReadyForReading:
-				    _FDToObject[FDs[i].fd]];
+				    [objects objectAtIndex: i]];
 
 			objc_autoreleasePoolPop(pool2);
 		}
@@ -230,12 +219,10 @@ removeObject(OFPollKernelEventObserver *self, id object, int fd, short events)
 			if ([_delegate respondsToSelector:
 			    @selector(objectIsReadyForWriting:)])
 				[_delegate objectIsReadyForWriting:
-				    _FDToObject[FDs[i].fd]];
+				    [objects objectAtIndex: i]];
 
 			objc_autoreleasePoolPop(pool2);
 		}
-
-		FDs[i].revents = 0;
 	}
 
 	objc_autoreleasePoolPop(pool);
