@@ -125,10 +125,10 @@ processBlock(uint32_t *state, uint32_t *buffer)
 	self = [super init];
 
 	@try {
-		_iVarsData = [[OFSecureData alloc]
-			    initWithCount: sizeof(*_iVars)
+		_ivarsData = [[OFSecureData alloc]
+			    initWithCount: sizeof(*_ivars)
 		    allowsSwappableMemory: allowsSwappableMemory];
-		_iVars = _iVarsData.mutableItems;
+		_ivars = _ivarsData.mutableItems;
 		_allowsSwappableMemory = allowsSwappableMemory;
 
 		[self of_resetState];
@@ -152,7 +152,7 @@ processBlock(uint32_t *state, uint32_t *buffer)
 
 - (void)dealloc
 {
-	objc_release(_iVarsData);
+	objc_release(_ivarsData);
 
 	[super dealloc];
 }
@@ -171,8 +171,8 @@ processBlock(uint32_t *state, uint32_t *buffer)
 {
 	OFSHA1Hash *copy = [[OFSHA1Hash alloc] of_init];
 
-	copy->_iVarsData = [_iVarsData copy];
-	copy->_iVars = copy->_iVarsData.mutableItems;
+	copy->_ivarsData = [_ivarsData copy];
+	copy->_ivars = copy->_ivarsData.mutableItems;
 	copy->_allowsSwappableMemory = _allowsSwappableMemory;
 	copy->_calculated = _calculated;
 
@@ -181,11 +181,11 @@ processBlock(uint32_t *state, uint32_t *buffer)
 
 - (void)of_resetState
 {
-	_iVars->state[0] = 0x67452301;
-	_iVars->state[1] = 0xEFCDAB89;
-	_iVars->state[2] = 0x98BADCFE;
-	_iVars->state[3] = 0x10325476;
-	_iVars->state[4] = 0xC3D2E1F0;
+	_ivars->state[0] = 0x67452301;
+	_ivars->state[1] = 0xEFCDAB89;
+	_ivars->state[2] = 0x98BADCFE;
+	_ivars->state[3] = 0x10325476;
+	_ivars->state[4] = 0xC3D2E1F0;
 }
 
 - (void)updateWithBuffer: (const void *)buffer_ length: (size_t)length
@@ -199,24 +199,24 @@ processBlock(uint32_t *state, uint32_t *buffer)
 	if (length > SIZE_MAX / 8)
 		@throw [OFOutOfRangeException exception];
 
-	_iVars->bits += (length * 8);
+	_ivars->bits += (length * 8);
 
 	while (length > 0) {
-		size_t min = 64 - _iVars->bufferLength;
+		size_t min = 64 - _ivars->bufferLength;
 
 		if (min > length)
 			min = length;
 
-		memcpy(_iVars->buffer.bytes + _iVars->bufferLength,
+		memcpy(_ivars->buffer.bytes + _ivars->bufferLength,
 		    buffer, min);
-		_iVars->bufferLength += min;
+		_ivars->bufferLength += min;
 
 		buffer += min;
 		length -= min;
 
-		if (_iVars->bufferLength == 64) {
-			processBlock(_iVars->state, _iVars->buffer.words);
-			_iVars->bufferLength = 0;
+		if (_ivars->bufferLength == 64) {
+			processBlock(_ivars->state, _ivars->buffer.words);
+			_ivars->bufferLength = 0;
 		}
 	}
 }
@@ -226,7 +226,7 @@ processBlock(uint32_t *state, uint32_t *buffer)
 	if (!_calculated)
 		@throw [OFHashNotCalculatedException exceptionWithObject: self];
 
-	return (const unsigned char *)_iVars->state;
+	return (const unsigned char *)_ivars->state;
 }
 
 - (void)calculate
@@ -235,32 +235,32 @@ processBlock(uint32_t *state, uint32_t *buffer)
 		@throw [OFHashAlreadyCalculatedException
 		    exceptionWithObject: self];
 
-	_iVars->buffer.bytes[_iVars->bufferLength] = 0x80;
-	OFZeroMemory(_iVars->buffer.bytes + _iVars->bufferLength + 1,
-	    64 - _iVars->bufferLength - 1);
+	_ivars->buffer.bytes[_ivars->bufferLength] = 0x80;
+	OFZeroMemory(_ivars->buffer.bytes + _ivars->bufferLength + 1,
+	    64 - _ivars->bufferLength - 1);
 
-	if (_iVars->bufferLength >= 56) {
-		processBlock(_iVars->state, _iVars->buffer.words);
-		OFZeroMemory(_iVars->buffer.bytes, 64);
+	if (_ivars->bufferLength >= 56) {
+		processBlock(_ivars->state, _ivars->buffer.words);
+		OFZeroMemory(_ivars->buffer.bytes, 64);
 	}
 
-	_iVars->buffer.words[14] =
-	    OFToBigEndian32((uint32_t)(_iVars->bits >> 32));
-	_iVars->buffer.words[15] =
-	    OFToBigEndian32((uint32_t)(_iVars->bits & 0xFFFFFFFF));
+	_ivars->buffer.words[14] =
+	    OFToBigEndian32((uint32_t)(_ivars->bits >> 32));
+	_ivars->buffer.words[15] =
+	    OFToBigEndian32((uint32_t)(_ivars->bits & 0xFFFFFFFF));
 
-	processBlock(_iVars->state, _iVars->buffer.words);
-	OFZeroMemory(&_iVars->buffer, sizeof(_iVars->buffer));
-	byteSwapVectorIfLE(_iVars->state, 5);
+	processBlock(_ivars->state, _ivars->buffer.words);
+	OFZeroMemory(&_ivars->buffer, sizeof(_ivars->buffer));
+	byteSwapVectorIfLE(_ivars->state, 5);
 	_calculated = true;
 }
 
 - (void)reset
 {
 	[self of_resetState];
-	_iVars->bits = 0;
-	OFZeroMemory(&_iVars->buffer, sizeof(_iVars->buffer));
-	_iVars->bufferLength = 0;
+	_ivars->bits = 0;
+	OFZeroMemory(&_ivars->buffer, sizeof(_ivars->buffer));
+	_ivars->bufferLength = 0;
 	_calculated = false;
 }
 @end
