@@ -31,7 +31,7 @@ OF_ASSUME_NONNULL_BEGIN
 @interface OFSHA224Or256Hash: OFObject <OFCryptographicHash>
 {
 @private
-	OFSecureData *_iVarsData;
+	OFSecureData *_ivarsData;
 @protected
 	struct {
 		uint32_t state[8];
@@ -41,7 +41,7 @@ OF_ASSUME_NONNULL_BEGIN
 			uint32_t words[64];
 		} buffer;
 		size_t bufferLength;
-	} *_iVars;
+	} *_ivars;
 @private
 	bool _allowsSwappableMemory;
 	bool _calculated;
