@@ -36,13 +36,13 @@ static const size_t digestSize = 64;
 
 - (void)of_resetState
 {
-	_iVars->state[0] = 0x6A09E667F3BCC908;
-	_iVars->state[1] = 0xBB67AE8584CAA73B;
-	_iVars->state[2] = 0x3C6EF372FE94F82B;
-	_iVars->state[3] = 0xA54FF53A5F1D36F1;
-	_iVars->state[4] = 0x510E527FADE682D1;
-	_iVars->state[5] = 0x9B05688C2B3E6C1F;
-	_iVars->state[6] = 0x1F83D9ABFB41BD6B;
-	_iVars->state[7] = 0x5BE0CD19137E2179;
+	_ivars->state[0] = 0x6A09E667F3BCC908;
+	_ivars->state[1] = 0xBB67AE8584CAA73B;
+	_ivars->state[2] = 0x3C6EF372FE94F82B;
+	_ivars->state[3] = 0xA54FF53A5F1D36F1;
+	_ivars->state[4] = 0x510E527FADE682D1;
+	_ivars->state[5] = 0x9B05688C2B3E6C1F;
+	_ivars->state[6] = 0x1F83D9ABFB41BD6B;
+	_ivars->state[7] = 0x5BE0CD19137E2179;
 }
 @end
