@@ -131,10 +131,10 @@
 								 errNo: errno];
 
 	_canBlock = canBlock;
-#elif defined(OF_WINDOWS)
-	u_long v = !canBlock;
+#elif defined(OF_WINDOWS) || defined(OF_AMIGAOS)
+	unsigned long v = !canBlock;
 
-	if (ioctlsocket(_socket, FIONBIO, &v) == SOCKET_ERROR)
+	if (ioctlsocket(_socket, FIONBIO, (void *)&v) == SOCKET_ERROR)
 		@throw [OFSetOptionFailedException
 		    exceptionWithObject: self
 				  errNo: _OFSocketErrNo()];
