@@ -288,7 +288,8 @@ typedef OFString *_Nullable (^OFStreamStringWrittenHandler)(OFStream *stream,
 	bool _buffersWrites, _waitingForDelimiter;
 @private
 	uintptr_t _encoding;
-	OF_RESERVE_IVARS(OFStream, 3)
+	uintptr_t _allowsLossyEncoding;
+	OF_RESERVE_IVARS(OFStream, 2)
 }
 
 /**
@@ -314,6 +315,16 @@ typedef OFString *_Nullable (^OFStreamStringWrittenHandler)(OFStream *stream,
  */
 @property (nonatomic) OFStringEncoding encoding;
 
+/**
+ * @brief Whether the stream allows the specified encoding to be lossy.
+ *
+ * This means that if a character cannot be represented in the stream's
+ * encoding, a `?` will be written instead. This is mostly useful for writing
+ * to standard output, as a user would prefer unrepresentable characters
+ * replaced over an exception and seeing nothing.
+ */
+@property (nonatomic) bool allowsLossyEncoding;
+
 /**
  * @brief Whether the stream can block.
  *
