@@ -535,9 +535,13 @@ retry_2:
 			  encoding: (OFStringEncoding)encoding
 {
 	OFString *ret;
-	char *buffer = OFAllocMemory(length + 1, 1);
-	buffer[length] = 0;
+	char *buffer;
 
+	if (SIZE_MAX - length < 1)
+		@throw [OFOutOfRangeException exception];
+
+	buffer = OFAllocMemory(length + 1, 1);
+	buffer[length] = 0;
 	@try {
 		[self readIntoBuffer: buffer exactLength: length];
 		ret = [OFString stringWithCString: buffer encoding: encoding];
@@ -685,6 +689,9 @@ retry:
 
 		/* There was no newline or \0 */
 		if (bufferLength > 0) {
+			if (SIZE_MAX - _readBufferLength < bufferLength)
+				@throw [OFOutOfRangeException exception];
+
 			readBuffer = OFAllocMemory(
 			    _readBufferLength + bufferLength, 1);
 
@@ -1002,6 +1009,9 @@ retry:
 
 		/* No \0 was found */
 		if (bufferLength > 0) {
+			if (SIZE_MAX - _readBufferLength < bufferLength)
+				@throw [OFOutOfRangeException exception];
+
 			readBuffer = OFAllocMemory(
 			    _readBufferLength + bufferLength, 1);
 
@@ -1580,8 +1590,12 @@ retry:
 	pool = objc_autoreleasePoolPush();
 	length = [string cStringLengthWithEncoding: encoding];
 
-	[self writeBuffer: [string cStringWithEncoding: encoding]
-		   length: length];
+	if (_allowsLossyEncoding)
+		[self writeBuffer: [string lossyCStringWithEncoding: encoding]
+			   length: length];
+	else
+		[self writeBuffer: [string cStringWithEncoding: encoding]
+			   length: length];
 
 	objc_autoreleasePoolPop(pool);
 }
@@ -1599,8 +1613,13 @@ retry:
 	buffer = OFAllocMemory(stringLength + 1, 1);
 
 	@try {
-		memcpy(buffer, [string cStringWithEncoding: encoding],
-		    stringLength);
+		if (_allowsLossyEncoding)
+			memcpy(buffer, [string lossyCStringWithEncoding:
+			    encoding], stringLength);
+		else
+			memcpy(buffer, [string cStringWithEncoding: encoding],
+			    stringLength);
+
 		buffer[stringLength] = '\n';
 
 		[self writeBuffer: buffer length: stringLength + 1];
@@ -1663,6 +1682,16 @@ retry:
 	_encoding = encoding;
 }
 
+- (bool)allowsLossyEncoding
+{
+	return _allowsLossyEncoding;
+}
+
+- (void)setAllowsLossyEncoding: (bool)allowsLossyEncoding
+{
+	_allowsLossyEncoding = allowsLossyEncoding;
+}
+
 - (bool)canBlock
 {
 	return _canBlock;
