@@ -285,6 +285,10 @@ initOperatingSystemVersion(void)
 # endif
 #elif defined(OF_ANDROID)
 	/* TODO */
+#elif defined(OF_AMIGAOS4)
+	operatingSystemVersion = [[OFString alloc]
+	    initWithFormat: @"Kickstart %u.%u",
+			    SysBase->lib_Version, SysBase->lib_Revision];
 #elif defined(OF_AMIGAOS)
 	operatingSystemVersion = [[OFString alloc]
 	    initWithFormat: @"Kickstart %u.%u",
@@ -1039,7 +1043,7 @@ cpucfg(uint32_t word)
 	uint32_t supportsAltiVec;
 
 	if (NewGetSystemAttrs(&supportsAltiVec, sizeof(supportsAltiVec),
-	    SYSTEMINFOTYPE_PPC_ALTIVEC, TAG_DONE) > 0)
+	    SYSTEMINFOTYPE_PPC_ALTIVEC, TAG_END) > 0)
 		return supportsAltiVec;
 # endif
 
