@@ -59,9 +59,6 @@ OF_ASSUME_NONNULL_BEGIN
 /**
  * @brief Returns the certificate chain from the PEM file at the specified IRI.
  *
- * @note This is not available on iOS when using Secure Transport! Use
- *	 @ref certificateChainFromPKCS12FileAtIRI:passphrase: instead!
- *
  * @param certificatesIRI The IRI to the PEM file with the certificate chain
  * @param privateKeyIRI An optional IRI to the PEM file with the private key or
  *			`nil`
@@ -79,7 +76,7 @@ OF_ASSUME_NONNULL_BEGIN
  * @brief Returns the certificate chain from the PKCS #12 file at the specified
  *	  IRI.
  *
- * @note This is not available when using mbedTLS! Use
+ * @note This is not available when using Mbed TLS! Use
  *	 @ref certificateChainFromPEMFileAtIRI:privateKeyIRI: instead!
  *
  * @param IRI The IRI to the PKCS #12 file with the certificate chain
