@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -120,6 +120,14 @@ OF_ASSUME_NONNULL_BEGIN
 		      namespace: (nullable OFString *)nameSpace
 		    stringValue: (nullable OFString *)stringValue;
 
+/**
+ * @brief Creates a new XML element with the specified element.
+ *
+ * @param element The element for the element
+ * @return A new autoreleased OFXMLElement with the specified element
+ */
++ (instancetype)elementWithElement: (OFXMLElement *)element;
+
 /**
  * @brief Parses the string and returns an OFXMLElement for it.
  *
@@ -194,6 +202,15 @@ OF_ASSUME_NONNULL_BEGIN
 		   namespace: (nullable OFString *)nameSpace
 		 stringValue: (nullable OFString *)stringValue;
 
+/**
+ * @brief Initializes an already allocated OFXMLElement with the specified
+ *	  element.
+ *
+ * @param element The element for the element
+ * @return An initialized OFXMLElement with the specified element
+ */
+- (instancetype)initWithElement: (OFXMLElement *)element;
+
 /**
  * @brief Parses the string and initializes an already allocated OFXMLElement
  *	  with it.
