@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2026 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -208,7 +208,8 @@ extern "C" {
 extern OFString *OFStrError(int errNo);
 
 #ifdef OF_WINDOWS
-extern OFString *_OFWindowsStatusToString(LSTATUS status) OF_VISIBILITY_HIDDEN;
+extern OFString *_OFWindowsStatusToString(LSTATUS status)
+    OF_VISIBILITY_INTERNAL;
 #endif
 #ifdef __cplusplus
 }
