@@ -35,7 +35,12 @@
 {
 	self = [super init];
 
-	_IRI = objc_retain(IRI);
+	@try {
+		_IRI = [IRI copy];
+	} @catch (id e) {
+		objc_release(self);
+		@throw e;
+	}
 
 	return self;
 }
