@@ -50,7 +50,7 @@ _object_isTaggedPointer_fast(id object)
 	return pointer & 1;
 }
 
-/* Inlined and unncessary checks dropped for performance. */
+/* Inlined and unnecessary checks dropped for performance. */
 static OF_INLINE Class
 _object_getClass_fast(id object_)
 {
@@ -308,16 +308,16 @@ objc_loadWeakRetained(id *object)
 	if (*object != nil && _objc_hashtable_get(hashtable, *object) != NULL)
 		value = *object;
 
+	if (!class_respondsToSelector(object_getClass(value),
+	    @selector(retainWeakReference)) || ![value retainWeakReference])
+		value = nil;
+
 #ifdef OF_HAVE_THREADS
 	if (OFSpinlockUnlock(&spinlock) != 0)
 		_OBJC_ERROR("Failed to unlock spinlock!");
 #endif
 
-	if (class_respondsToSelector(object_getClass(value),
-	    @selector(retainWeakReference)) && [value retainWeakReference])
-		return value;
-
-	return nil;
+	return value;
 }
 
 id
