@@ -535,6 +535,10 @@ isSubclassOfClass(Class class, Class superclass)
 		}
 	}
 
+#ifdef OF_AMIGAOS4
+	/* Work around a bug in AmigaOS 4 terminal. */
+	OFStdOut.foregroundColor = [OFColor purple];
+#endif
 	OFStdOut.bold = true;
 	OFStdOut.foregroundColor = [OFColor fuchsia];
 	[OFStdOut writeFormat: @"%zu", numSucceeded];
