@@ -265,7 +265,7 @@ printProfile(id <OHGameControllerProfile> profile)
 		[OFStdOut clear];
 	}
 
-	[OFStdOut setCursorPosition: OFMakePoint(0, 0)];
+	[OFStdOut setCursorPosition: OFMakePoint(0.0f, 0.0f)];
 
 	for (OHGameController *controller in _controllers) {
 		id <OHGameControllerProfile> profile = controller.profile;
@@ -329,7 +329,7 @@ printProfile(id <OHGameControllerProfile> profile)
 		objc_autoreleasePoolPop(pool);
 	}
 #else
-	[OFTimer scheduledTimerWithTimeInterval: 1.f / 60.f
+	[OFTimer scheduledTimerWithTimeInterval: 1.0f / 60.0f
 					 target: self
 				       selector: @selector(updateOutput)
 					repeats: true];
