@@ -21,6 +21,7 @@
 
 #include <math.h>
 
+#import "OFColor.h"
 #import "OFDate.h"
 #import "OFStdIOStream.h"
 #import "OFTimer.h"
@@ -102,7 +103,7 @@ static const OFTimeInterval updateInterval = 0.1;
 
 - (void)_drawProgress
 {
-	float bars, percent;
+	float bars, progress, percent;
 	int columns, barWidth;
 
 	if ((columns = OFStdErr.columns) >= 0) {
@@ -113,10 +114,12 @@ static const OFTimeInterval updateInterval = 0.1;
 	} else
 		barWidth = 43;
 
-	bars = (float)(_resumedFrom + _received) /
-	    (float)(_resumedFrom + _length) * barWidth;
-	percent = (float)(_resumedFrom + _received) /
-	    (float)(_resumedFrom + _length) * 100;
+	progress = (float)(_resumedFrom + _received) /
+	    (float)(_resumedFrom + _length);
+	bars = progress * barWidth;
+	percent = progress * 100.0f;
+
+	[OFStdErr setProgressIndicator: progress];
 
 	if (_useUnicode) {
 		[OFStdErr writeString: @"\r  ▕"];
@@ -147,7 +150,7 @@ static const OFTimeInterval updateInterval = 0.1;
 				[OFStdErr writeString: @" "];
 		}
 
-		[OFStdErr writeFormat: @"▏ %,6.2f%% ", percent];
+		[OFStdErr writeString: @"▏ "];
 	} else {
 		[OFStdErr writeString: @"\r  ["];
 
@@ -169,10 +172,20 @@ static const OFTimeInterval updateInterval = 0.1;
 				[OFStdErr writeString: @" "];
 		}
 
-		[OFStdErr writeFormat: @"] %,6.2f%% ", percent];
+		[OFStdErr writeString: @"] "];
 	}
 
-	if (percent == 100) {
+	OFStdErr.foregroundColor = [OFColor
+	    colorWithRed: (percent < 50.0f
+			      ? 1.0f : 1.0f - ((percent - 50.0f) / 50.0f))
+		   green: (percent < 50.0f ? percent / 50.0f : 1.0f)
+		    blue: 0.0f
+		   alpha: 1.0f];
+	[OFStdErr writeFormat: @"%,6.2f%%", percent];
+	OFStdErr.foregroundColor = nil;
+	[OFStdErr writeString: @" "];
+
+	if (progress >= 1.0f) {
 		double timeInterval = -_startDate.timeIntervalSinceNow;
 
 		_BPS = (float)_received / (float)timeInterval;
@@ -321,6 +334,8 @@ static const OFTimeInterval updateInterval = 0.1;
 
 - (void)stop
 {
+	[OFStdErr removeProgressIndicator];
+
 	[_drawTimer invalidate];
 	[_BPSTimer invalidate];
 
